package com.fossgalaxy.games.ggj2017;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Arrays;
import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JToolBar;

import com.fossgalaxy.games.rts.App;
import com.fossgalaxy.games.rts.GameState;
import com.fossgalaxy.games.rts.ai.Controller;
import com.fossgalaxy.games.rts.entity.Entity;
import com.fossgalaxy.games.rts.entity.EntityType;
import com.fossgalaxy.games.rts.io.SettingsIO;
import com.fossgalaxy.games.rts.order.Order;
import com.fossgalaxy.games.rts.order.OrderProcessor;
import com.fossgalaxy.games.rts.ui.GameAction;
import com.fossgalaxy.games.rts.ui.GameModel;
import com.fossgalaxy.games.rts.ui.GameView;
import com.fossgalaxy.games.rts.ui.KeyboardController;
import com.fossgalaxy.games.rts.ui.MouseController;
import com.fossgalaxy.games.rts.ui.UIController;

/**
 * Hello world!
 *
 */
public class JamApp 
{
    public static void main( String[] args )
    {

        //create the initial game state
        SettingsIO io = new SettingsIO();
        GameState state = io.buildGame("map.json");

        //create the players
        //TODO clone, without it all moves WILL be applied twice.
        GameState playerState0 = new GameState(state);
        GameState playerState1 = new GameState(state);
        
        Controller[] players = new Controller[]{
                JamApp.buildUI(playerState0, "player 0"), JamApp.buildUI(playerState1, "player 1")
        };

        //create the thing that's doing the turn tracking.
        OrderProcessor processor = new OrderProcessor(state, 2);
        int MAX_TURNS = 100;

        //turn loop
        //TODO this might be better inside the turn processor?
        for (int i=0; i<MAX_TURNS; i++) {
            processor.setupTurn();

            //figure out who's turn it is.
            int playerID = processor.getCurrentPlayer();
            Controller player = players[playerID];

            //process the controller's orders
            GameState curr = new GameState(state);
            Map<UUID, Order> orders = player.doTurn(playerID, curr);
            processor.doOrderBulk(orders);

            processor.finishTurn();
        }

    }

    public static Collection<GameAction> getPossibleActions() {
    	return Arrays.asList(new MoveAction(), new RangeAttackAction());
    }
    
    public static Collection<GameAction> getGameActions(Entity entity){
    	return getGameActions(entity, getPossibleActions());
    }
    
    public static Collection<GameAction> getGameActions(Entity entity, Collection<GameAction> actions) {
    	return actions.stream().filter(act -> act.isPossible(entity)).collect(Collectors.toList());
    }
    
    public static Controller buildUI(GameState state, String title) {

        JFrame frame = new JFrame(title);
        frame.setPreferredSize(new Dimension(800, 600));
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //create the UI stuff
        GameModel model = new GameModel(state);
        GameView view = new GameView(model);
        view.addMouseListener(new MouseController(view, model));

        view.addKeyListener(new KeyboardController(model, view));
        view.setFocusable(true);
        view.requestFocus();
        frame.add(view);
        
        JToolBar toolbar = new JToolBar();
        
        JButton moveBtn = new JButton("Move");
        moveBtn.addActionListener(a -> {
        	model.setAction(new MoveAction());
        	view.repaint();
        });
        
        toolbar.add(moveBtn);
        
        JButton rangedAttackBtn = new JButton("Attack");
        rangedAttackBtn.addActionListener(a -> {
        	model.setAction(new RangeAttackAction());
        	view.repaint();
        });
        toolbar.add(rangedAttackBtn);
        
        toolbar.add(new JButton("Deploy Pigeon"));
        frame.add(toolbar, BorderLayout.SOUTH);
        
        frame.pack();
        frame.setVisible(true);

        UIController controller = new UIController(view, model);
        controller.addPossibleActions(getPossibleActions());
        
        return new UIController(view, model);
    }
}
